/*
 * Copyright (C) 2022 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.docker.complex;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.fail;

import java.io.File;
import java.util.UUID;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.openepics.names.docker.ITUtil;
import org.openepics.names.docker.ITUtilStructureElement;
import org.openepics.names.rest.beans.Type;
import org.openepics.names.rest.beans.element.StructureElement;
import org.openepics.names.rest.beans.element.StructureElementCommand;
import org.openepics.names.util.EnumUtil.StructureChoice;
import org.testcontainers.containers.DockerComposeContainer;
import org.testcontainers.containers.wait.strategy.Wait;
import org.testcontainers.junit.jupiter.Container;
import org.testcontainers.junit.jupiter.Testcontainers;

/**
 * Integration tests for Naming and PostgreSQL that make use of existing dockerization
 * with docker-compose.yml / Dockerfile.
 *
 * <p>
 * Focus of this class is structures endpoint and system.
 * In particular, purpose is to test arrays of structures.
 * </p>
 *
 * @author Lars Johansson
 */
@Testcontainers
class StructuresMultipleIT {

    @Container
    public static final DockerComposeContainer<?> ENVIRONMENT =
        new DockerComposeContainer<>(new File("docker-compose-it-db-schema-migration.yml"))
            .waitingFor(ITUtil.NAMING, Wait.forLogMessage(".*Started NamingApplication.*", 1));

    private static UUID systemGroupUuid = null;
    private static UUID systemUuid = null;
    private static UUID subsystemUuid = null;

    private static UUID disciplineUuid = null;
    private static UUID deviceGroupUuid = null;
    @SuppressWarnings("unused")
    private static UUID deviceTypeUuid = null;

    @BeforeAll
    public static void initAll() {
        // init system group, system, discipline, device group

        StructureElementCommand structureElement  = null;
        StructureElement approvedStructureElement = null;

        structureElement = new StructureElementCommand(
                null, Type.SYSTEMGROUP, null,
                "name", "Sg",
                "description", "comment");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        systemGroupUuid = approvedStructureElement.getUuid();

        structureElement = new StructureElementCommand(
                null, Type.SYSTEM, systemGroupUuid,
                "name", "Sys",
                "description", "comment");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        systemUuid = approvedStructureElement.getUuid();

        structureElement = new StructureElementCommand(
                null, Type.SUBSYSTEM, systemUuid,
                "name", "Sub",
                "description", "comment");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        subsystemUuid = approvedStructureElement.getUuid();

        structureElement = new StructureElementCommand(
                null, Type.DISCIPLINE, null,
                "name", "Di",
                "description", "comment");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        disciplineUuid = approvedStructureElement.getUuid();

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, disciplineUuid,
                "name", null,
                "description", "comment");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        deviceGroupUuid = approvedStructureElement.getUuid();

        structureElement = new StructureElementCommand(
                null, Type.DEVICETYPE, deviceGroupUuid,
                "name", "Dt",
                "description", "comment");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        deviceTypeUuid = approvedStructureElement.getUuid();
    }

    @Test
    void checkCreate() {
        // purpose
        //     test conditions for create subsystem
        //         not create itself
        //
        // what - combination of
        //     create      create structures
        //     read        exists in structure
        //     read        is valid to create
        //     read        validate create
        //
        // validate create
        //     type
        //     parent
        //     name
        //     mnemonic
        //     description
        //     comment

        try {
            StructureElementCommand[] structureElements = null;
            UUID uuid = null;
            String value = null;

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "Sg",    Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgChc", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "Sg",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgChc", Boolean.TRUE);

            ITUtilStructureElement.assertExists(Type.SYSTEM, "Sys",    Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM, "SysChc", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM, "Sys",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM, "SysChc", Boolean.TRUE);

            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-Sub", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-SubChc",  Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, "Sys-Sub",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, "Sys-SubChc", Boolean.TRUE);

            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "Di",   Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "DiChc", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE, "Di",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE, "DiChc", Boolean.TRUE);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Grp",   Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpChc", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Grp",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpChc", Boolean.FALSE);

            ITUtilStructureElement.assertExists(Type.DEVICETYPE, "Di-Dt",    Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE, "Di-DtChc", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE, "Di-Dt",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE, "Di-DtChc", Boolean.TRUE);

            structureElements = new StructureElementCommand[] {
                    new StructureElementCommand(null, Type.SYSTEMGROUP, null,            "name", "SgChc",  "description", "comment"),
                    new StructureElementCommand(null, Type.SYSTEM,      systemGroupUuid, "name", "SysChc", "description", "comment"),
                    new StructureElementCommand(null, Type.SUBSYSTEM,   systemUuid,      "name", "SubChc", "description", "comment"),
                    new StructureElementCommand(null, Type.DISCIPLINE,  null,            "name", "DiChc",  "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICEGROUP, disciplineUuid,  "name", "",      "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICETYPE,  deviceGroupUuid, "name", "DtChc",  "description", "comment")
            };
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.REJECT,  Boolean.FALSE);

            uuid = subsystemUuid;
            structureElements[1].setUuid(uuid);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE, Boolean.FALSE);
            structureElements[1].setUuid(null);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE, Boolean.TRUE);

            structureElements[1].setType(null);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE, Boolean.FALSE);
            structureElements[1].setType(Type.SYSTEMGROUP);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE, Boolean.FALSE);
            structureElements[1].setType(Type.SUBSYSTEM);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE, Boolean.FALSE);
            structureElements[1].setType(Type.SYSTEM);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE, Boolean.TRUE);

            structureElements[1].setParent(null);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE, Boolean.FALSE);
            structureElements[1].setParent(systemUuid);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE, Boolean.FALSE);
            structureElements[1].setParent(subsystemUuid);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE, Boolean.FALSE);
            structureElements[1].setParent(systemGroupUuid);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE, Boolean.TRUE);

            value = structureElements[1].getName();
            structureElements[1].setName(null);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE, Boolean.FALSE);
            structureElements[1].setName(value);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE, Boolean.TRUE);

            value = structureElements[1].getDescription();
            structureElements[1].setDescription(null);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE, Boolean.FALSE);
            structureElements[1].setDescription(value);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE, Boolean.TRUE);

            value = structureElements[1].getComment();
            structureElements[1].setComment(null);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE, Boolean.FALSE);
            structureElements[1].setComment(value);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE, Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void createApprove() {
        // purpose
        //     test create and approve subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        exists in structure
        //     read        is valid to create
        //     read        validate create
        //     read        validate approve
        //     patch       approve structures
        //
        // note
        //     create in order to approve

        try {
            StructureElementCommand[] structureElements = null;
            StructureElement[] createdStructureElements = null;
            StructureElement[] approvedStructureElements = null;

            structureElements = new StructureElementCommand[] {
                    new StructureElementCommand(null, Type.SYSTEMGROUP, null,            "name", "SgCa",  "description", "comment"),
                    new StructureElementCommand(null, Type.SYSTEM,      systemGroupUuid, "name", "SysCa", "description", "comment"),
                    new StructureElementCommand(null, Type.SUBSYSTEM,   systemUuid,      "name", "SubCa", "description", "comment"),
                    new StructureElementCommand(null, Type.DISCIPLINE,  null,            "name", "DiCa",  "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICEGROUP, disciplineUuid,  "name", "",      "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICETYPE,  deviceGroupUuid, "name", "DtCa",  "description", "comment")
            };

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgCa",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysCa",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubCa", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiCa",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpCa",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtCa",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgCa",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysCa",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubCa", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiCa",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpCa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtCa",   Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.REJECT,  Boolean.FALSE);

            createdStructureElements = ITUtilStructureElement.assertCreate(structureElements);
            assertNotNull(createdStructureElements);
            assertEquals(6, createdStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgCa",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysCa",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubCa", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiCa",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpCa",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtCa",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgCa",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysCa",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubCa", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiCa",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpCa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtCa",   Boolean.TRUE);

            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.APPROVE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.CANCEL,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.REJECT,  Boolean.TRUE);

            approvedStructureElements = ITUtilStructureElement.assertApprove(createdStructureElements);
            assertNotNull(approvedStructureElements);
            assertEquals(6, approvedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgCa",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysCa",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubCa", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiCa",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpCa",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtCa",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgCa",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysCa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubCa", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiCa",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpCa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtCa",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.REJECT,  Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void createCancel() {
        // purpose
        //     test create and cancel subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        exists in structure
        //     read        is valid to create
        //     read        validate create
        //     read        validate cancel
        //     patch       cancel structures
        //
        // note
        //     create in order to cancel

        try {
            StructureElementCommand[] structureElements = null;
            StructureElement[] createdStructureElements = null;
            StructureElement[] cancelledStructureElements = null;

            structureElements = new StructureElementCommand[] {
                    new StructureElementCommand(null, Type.SYSTEMGROUP, null,            "name", "SgCc",  "description", "comment"),
                    new StructureElementCommand(null, Type.SYSTEM,      systemGroupUuid, "name", "SysCc", "description", "comment"),
                    new StructureElementCommand(null, Type.SUBSYSTEM,   systemUuid,      "name", "SubCc", "description", "comment"),
                    new StructureElementCommand(null, Type.DISCIPLINE,  null,            "name", "DiCc",  "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICEGROUP, disciplineUuid,  "name", "",      "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICETYPE,  deviceGroupUuid, "name", "DtCc",  "description", "comment")
            };

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgCc",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysCc",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubCc", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiCc",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpCc",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtCc",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgCc",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysCc",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubCc", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiCc",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpCc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtCc",   Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.REJECT,  Boolean.FALSE);

            createdStructureElements = ITUtilStructureElement.assertCreate(structureElements);
            assertNotNull(createdStructureElements);
            assertEquals(6, createdStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgCc",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysCc",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubCc", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiCc",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpCc",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtCc",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgCc",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysCc",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubCc", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiCc",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpCc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtCc",   Boolean.TRUE);

            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.APPROVE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.CANCEL,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.REJECT,  Boolean.TRUE);

            cancelledStructureElements = ITUtilStructureElement.assertCancel(createdStructureElements);
            assertNotNull(cancelledStructureElements);
            assertEquals(6, cancelledStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgCc",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysCc",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubCc", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiCc",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpCc",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtCc",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgCc",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysCc",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubCc", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiCc",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpCc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtCc",   Boolean.TRUE);

            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.REJECT,  Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void createReject() {
        // purpose
        //     test create and reject subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate reject
        //     patch       reject structures
        //
        // note
        //     create in order to reject

        try {
            StructureElementCommand[] structureElements = null;
            StructureElement[] createdStructureElements = null;
            StructureElement[] rejectedStructureElements = null;

            structureElements = new StructureElementCommand[] {
                    new StructureElementCommand(null, Type.SYSTEMGROUP, null,            "name", "SgCr",  "description", "comment"),
                    new StructureElementCommand(null, Type.SYSTEM,      systemGroupUuid, "name", "SysCr", "description", "comment"),
                    new StructureElementCommand(null, Type.SUBSYSTEM,   systemUuid,      "name", "SubCr", "description", "comment"),
                    new StructureElementCommand(null, Type.DISCIPLINE,  null,            "name", "DiCr",  "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICEGROUP, disciplineUuid,  "name", "",      "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICETYPE,  deviceGroupUuid, "name", "DtCr",  "description", "comment")
            };

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgCr",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysCr",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubCr", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiCr",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpCr",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtCr",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgCr",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysCr",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubCr", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiCr",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpCr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtCr",   Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.REJECT,  Boolean.FALSE);

            createdStructureElements = ITUtilStructureElement.assertCreate(structureElements);
            assertNotNull(createdStructureElements);
            assertEquals(6, createdStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgCr",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysCr",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubCr", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiCr",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpCr",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtCr",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgCr",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysCr",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubCr", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiCr",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpCr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtCr",   Boolean.TRUE);

            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.APPROVE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.CANCEL,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(createdStructureElements, StructureChoice.REJECT,  Boolean.TRUE);

            rejectedStructureElements = ITUtilStructureElement.assertReject(createdStructureElements);
            assertNotNull(rejectedStructureElements);
            assertEquals(6, rejectedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgCr",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysCr",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubCr", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiCr",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpCr",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtCr",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgCr",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysCr",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubCr", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiCr",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpCr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtCr",   Boolean.TRUE);

            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.REJECT,  Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void checkUpdate() {
        // purpose
        //     test conditions for update subsystem
        //         not update
        //
        // what - combination of
        //     create      create structures
        //     read        exists in structure
        //     read        is valid to create
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     patch       approve structures
        //
        // note
        //     create, approve in order to update
        //
        // validate update
        //     uuid
        //     type
        //     parent
        //     name
        //     mnemonic
        //     description
        //     comment

        try {
            StructureElementCommand[] structureElements = null;
            StructureElement[] approvedStructureElements = null;
            UUID uuid = null;
            String value = null;

            structureElements = new StructureElementCommand[] {
                    new StructureElementCommand(null, Type.SYSTEMGROUP, null,            "name", "SgCu",  "description", "comment"),
                    new StructureElementCommand(null, Type.SYSTEM,      systemGroupUuid, "name", "SysCu", "description", "comment"),
                    new StructureElementCommand(null, Type.SUBSYSTEM,   systemUuid,      "name", "SubCu", "description", "comment"),
                    new StructureElementCommand(null, Type.DISCIPLINE,  null,            "name", "DiCu",  "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICEGROUP, disciplineUuid,  "name", "",      "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICETYPE,  deviceGroupUuid, "name", "DtCu",  "description", "comment")
            };

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgCu",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysCu",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubCu", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiCu",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpCu",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtCu",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgCu",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysCu",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubCu", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiCu",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpCu",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtCu",   Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.REJECT,  Boolean.FALSE);

            approvedStructureElements = ITUtilStructureElement.assertCreateApprove(structureElements);
            assertNotNull(approvedStructureElements);
            assertEquals(6, approvedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgCu",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysCu",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubCu", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiCu",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpCu",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtCu",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgCu",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysCu",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubCu", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiCu",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpCu",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtCu",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.REJECT,  Boolean.FALSE);

            uuid = approvedStructureElements[1].getUuid();
            approvedStructureElements[1].setUuid(null);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE, Boolean.FALSE);
            approvedStructureElements[1].setUuid(uuid);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE, Boolean.TRUE);

            approvedStructureElements[1].setType(null);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE, Boolean.FALSE);
            approvedStructureElements[1].setType(Type.SYSTEMGROUP);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE, Boolean.FALSE);
            approvedStructureElements[1].setType(Type.SUBSYSTEM);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE, Boolean.FALSE);
            approvedStructureElements[1].setType(Type.SYSTEM);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE, Boolean.TRUE);

            approvedStructureElements[1].setParent(null);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE, Boolean.FALSE);
            approvedStructureElements[1].setParent(systemUuid);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE, Boolean.FALSE);
            approvedStructureElements[1].setParent(subsystemUuid);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE, Boolean.FALSE);
            approvedStructureElements[1].setParent(systemGroupUuid);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE, Boolean.TRUE);

            value = approvedStructureElements[1].getName();
            approvedStructureElements[1].setName(null);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE, Boolean.FALSE);
            approvedStructureElements[1].setName(value);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE, Boolean.TRUE);

            value = approvedStructureElements[1].getDescription();
            approvedStructureElements[1].setDescription(null);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE, Boolean.FALSE);
            approvedStructureElements[1].setDescription(value);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE, Boolean.TRUE);

            value = approvedStructureElements[1].getComment();
            approvedStructureElements[1].setComment(null);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE, Boolean.FALSE);
            approvedStructureElements[1].setComment(value);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE, Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void updateApprove() {
        // purpose
        //     test update and approve subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        exists in structure
        //     read        is valid to create
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     update      update structures
        //     patch       approve structures
        //
        // note
        //     create, approve in order to update, approve

        try {
            StructureElementCommand[] structureElements = null;
            StructureElement[] approvedStructureElements = null;
            StructureElement[] updatedStructureElements = null;

            structureElements = new StructureElementCommand[] {
                    new StructureElementCommand(null, Type.SYSTEMGROUP, null,            "name", "SgUa",  "description", "comment"),
                    new StructureElementCommand(null, Type.SYSTEM,      systemGroupUuid, "name", "SysUa", "description", "comment"),
                    new StructureElementCommand(null, Type.SUBSYSTEM,   systemUuid,      "name", "SubUa", "description", "comment"),
                    new StructureElementCommand(null, Type.DISCIPLINE,  null,            "name", "DiUa",  "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICEGROUP, disciplineUuid,  "name", "",      "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICETYPE,  deviceGroupUuid, "name", "DtUa",  "description", "comment")
            };

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgUa",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysUa",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubUa", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiUa",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpUa",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtUa",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgUa",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysUa",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubUa", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiUa",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpUa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtUa",   Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.REJECT,  Boolean.FALSE);

            approvedStructureElements = ITUtilStructureElement.assertCreateApprove(structureElements);
            assertNotNull(approvedStructureElements);
            assertEquals(6, approvedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgUa",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysUa",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubUa", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiUa",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpUa",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtUa",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgUa",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysUa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubUa", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiUa",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpUa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtUa",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.REJECT,  Boolean.FALSE);

            approvedStructureElements[0].setDescription("0");
            approvedStructureElements[1].setDescription("1");
            approvedStructureElements[2].setDescription("2");
            approvedStructureElements[3].setDescription("3");
            approvedStructureElements[4].setDescription("4");
            approvedStructureElements[5].setDescription("5");

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgUa",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysUa",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubUa", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiUa",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpUa",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtUa",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgUa",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysUa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubUa", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiUa",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpUa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtUa",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.REJECT,  Boolean.FALSE);

            updatedStructureElements = ITUtilStructureElement.assertUpdate(approvedStructureElements);
            assertNotNull(updatedStructureElements);
            assertEquals(6, updatedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgUa",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysUa",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubUa", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiUa",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpUa",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtUa",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgUa",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysUa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubUa", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiUa",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpUa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtUa",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.APPROVE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.CANCEL,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.REJECT,  Boolean.TRUE);

            approvedStructureElements = ITUtilStructureElement.assertApprove(updatedStructureElements);
            assertNotNull(approvedStructureElements);
            assertEquals(6, approvedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgUa",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysUa",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubUa", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiUa",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpUa",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtUa",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgUa",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysUa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubUa", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiUa",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpUa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtUa",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.REJECT,  Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void updateCancel() {
        // purpose
        //     test update and cancel subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        exists in structure
        //     read        is valid to create
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     read        validate cancel
        //     update      update structures
        //     patch       approve structures
        //     patch       cancel structures
        //
        // note
        //     create, approve in order to update, cancel

        try {
            StructureElementCommand[] structureElements = null;
            StructureElement[] approvedStructureElements = null;
            StructureElement[] updatedStructureElements = null;
            StructureElement[] cancelledStructureElements = null;

            structureElements = new StructureElementCommand[] {
                    new StructureElementCommand(null, Type.SYSTEMGROUP, null,            "name", "SgUc",  "description", "comment"),
                    new StructureElementCommand(null, Type.SYSTEM,      systemGroupUuid, "name", "SysUc", "description", "comment"),
                    new StructureElementCommand(null, Type.SUBSYSTEM,   systemUuid,      "name", "SubUc", "description", "comment"),
                    new StructureElementCommand(null, Type.DISCIPLINE,  null,            "name", "DiUc",  "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICEGROUP, disciplineUuid,  "name", "",      "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICETYPE,  deviceGroupUuid, "name", "DtUc",  "description", "comment")
            };

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgUc",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysUc",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubUc", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiUc",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpUc",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtUc",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgUc",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysUc",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubUc", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiUc",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpUc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtUc",   Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.REJECT,  Boolean.FALSE);

            approvedStructureElements = ITUtilStructureElement.assertCreateApprove(structureElements);
            assertNotNull(approvedStructureElements);
            assertEquals(6, approvedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgUc",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysUc",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubUc", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiUc",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpUc",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtUc",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgUc",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysUc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubUc", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiUc",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpUc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtUc",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.REJECT,  Boolean.FALSE);

            approvedStructureElements[0].setDescription("0");
            approvedStructureElements[1].setDescription("1");
            approvedStructureElements[2].setDescription("2");
            approvedStructureElements[3].setDescription("3");
            approvedStructureElements[4].setDescription("4");
            approvedStructureElements[5].setDescription("5");

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgUc",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysUc",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubUc", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiUc",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpUc",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtUc",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgUc",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysUc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubUc", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiUc",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpUc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtUc",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.REJECT,  Boolean.FALSE);

            updatedStructureElements = ITUtilStructureElement.assertUpdate(approvedStructureElements);
            assertNotNull(updatedStructureElements);
            assertEquals(6, updatedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgUc",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysUc",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubUc", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiUc",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpUc",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtUc",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgUc",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysUc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubUc", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiUc",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpUc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtUc",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.APPROVE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.CANCEL,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.REJECT,  Boolean.TRUE);

            cancelledStructureElements = ITUtilStructureElement.assertCancel(updatedStructureElements);
            assertNotNull(cancelledStructureElements);
            assertEquals(6, cancelledStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgUc",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysUc",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubUc", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiUc",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpUc",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtUc",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgUc",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysUc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubUc", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiUc",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpUc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtUc",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.REJECT,  Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void updateReject() {
        // purpose
        //     test update and reject subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        exists in structure
        //     read        is valid to create
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     read        validate reject
        //     update      update structures
        //     patch       approve structures
        //     patch       reject structures
        //
        // note
        //     create, approve in order to update, reject

        try {
            StructureElementCommand[] structureElements = null;
            StructureElement[] approvedStructureElements = null;
            StructureElement[] updatedStructureElements = null;
            StructureElement[] rejectedStructureElements = null;

            structureElements = new StructureElementCommand[] {
                    new StructureElementCommand(null, Type.SYSTEMGROUP, null,            "name", "SgUr",  "description", "comment"),
                    new StructureElementCommand(null, Type.SYSTEM,      systemGroupUuid, "name", "SysUr", "description", "comment"),
                    new StructureElementCommand(null, Type.SUBSYSTEM,   systemUuid,      "name", "SubUr", "description", "comment"),
                    new StructureElementCommand(null, Type.DISCIPLINE,  null,            "name", "DiUr",  "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICEGROUP, disciplineUuid,  "name", "",      "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICETYPE,  deviceGroupUuid, "name", "DtUr",  "description", "comment")
            };

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgUr",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysUr",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubUr", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiUr",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpUr",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtUr",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgUr",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysUr",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubUr", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiUr",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpUr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtUr",   Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.REJECT,  Boolean.FALSE);

            approvedStructureElements = ITUtilStructureElement.assertCreateApprove(structureElements);
            assertNotNull(approvedStructureElements);
            assertEquals(6, approvedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgUr",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysUr",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubUr", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiUr",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpUr",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtUr",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgUr",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysUr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubUr", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiUr",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpUr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtUr",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.REJECT,  Boolean.FALSE);

            approvedStructureElements[0].setDescription("0");
            approvedStructureElements[1].setDescription("1");
            approvedStructureElements[2].setDescription("2");
            approvedStructureElements[3].setDescription("3");
            approvedStructureElements[4].setDescription("4");
            approvedStructureElements[5].setDescription("5");

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgUr",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysUr",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubUr", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiUr",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpUr",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtUr",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgUr",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysUr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubUr", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiUr",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpUr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtUr",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.REJECT,  Boolean.FALSE);

            updatedStructureElements = ITUtilStructureElement.assertUpdate(approvedStructureElements);
            assertNotNull(updatedStructureElements);
            assertEquals(6, updatedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgUr",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysUr",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubUr", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiUr",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpUr",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtUr",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgUr",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysUr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubUr", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiUr",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpUr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtUr",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.APPROVE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.CANCEL,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(updatedStructureElements, StructureChoice.REJECT,  Boolean.TRUE);

            rejectedStructureElements = ITUtilStructureElement.assertReject(updatedStructureElements);
            assertNotNull(rejectedStructureElements);
            assertEquals(6, rejectedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgUr",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysUr",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubUr", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiUr",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpUr",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtUr",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgUr",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysUr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubUr", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiUr",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpUr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtUr",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.REJECT,  Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void checkDelete() {
        // purpose
        //     test conditions for delete subsystem
        //         not delete
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     patch       approve structures
        //
        // note
        //     create, approve in order to delete
        //
        // validate delete
        //     uuid
        //     type
        //     comment

        try {
            StructureElementCommand[] structureElements = null;
            StructureElement[] approvedStructureElements = null;
            UUID uuid = null;
            String value = null;

            structureElements = new StructureElementCommand[] {
                    new StructureElementCommand(null, Type.SYSTEMGROUP, null,            "name", "SgCd",  "description", "comment"),
                    new StructureElementCommand(null, Type.SYSTEM,      systemGroupUuid, "name", "SysCd", "description", "comment"),
                    new StructureElementCommand(null, Type.SUBSYSTEM,   systemUuid,      "name", "SubCd", "description", "comment"),
                    new StructureElementCommand(null, Type.DISCIPLINE,  null,            "name", "DiCd",  "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICEGROUP, disciplineUuid,  "name", "",      "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICETYPE,  deviceGroupUuid, "name", "DtCd",  "description", "comment")
            };

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgCd",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysCd",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubCd", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiCd",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpCd",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtCd",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgCd",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysCd",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubCd", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiCd",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpCd",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtCd",   Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.REJECT,  Boolean.FALSE);

            approvedStructureElements = ITUtilStructureElement.assertCreateApprove(structureElements);
            assertNotNull(approvedStructureElements);
            assertEquals(6, approvedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgCd",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysCd",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubCd", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiCd",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpCd",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtCd",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgCd",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysCd",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubCd", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiCd",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpCd",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtCd",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.REJECT,  Boolean.FALSE);

            uuid = approvedStructureElements[1].getUuid();
            approvedStructureElements[1].setUuid(null);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE, Boolean.FALSE);
            approvedStructureElements[1].setUuid(uuid);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE, Boolean.TRUE);

            approvedStructureElements[1].setType(null);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE, Boolean.FALSE);
            approvedStructureElements[1].setType(Type.SYSTEMGROUP);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE, Boolean.FALSE);
            approvedStructureElements[1].setType(Type.SUBSYSTEM);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE, Boolean.FALSE);
            approvedStructureElements[1].setType(Type.SYSTEM);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE, Boolean.TRUE);

            approvedStructureElements[1].setParent(null);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE, Boolean.TRUE);
            approvedStructureElements[1].setParent(systemUuid);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE, Boolean.TRUE);
            approvedStructureElements[1].setParent(subsystemUuid);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE, Boolean.TRUE);
            approvedStructureElements[1].setParent(systemGroupUuid);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE, Boolean.TRUE);

            value = approvedStructureElements[1].getName();
            approvedStructureElements[1].setName(null);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE, Boolean.TRUE);
            approvedStructureElements[1].setName(value);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE, Boolean.TRUE);

            value = approvedStructureElements[1].getDescription();
            approvedStructureElements[1].setDescription(null);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE, Boolean.TRUE);
            approvedStructureElements[1].setDescription(value);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE, Boolean.TRUE);

            value = approvedStructureElements[1].getComment();
            approvedStructureElements[1].setComment(null);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE, Boolean.FALSE);
            approvedStructureElements[1].setComment(value);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE, Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void deleteApprove() {
        // purpose
        //     test delete and approve subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        exists in structure
        //     read        is valid to create
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     delete      delete structures
        //     patch       approve structures
        //
        // note
        //     create, approve in order to delete, approve

        try {
            StructureElementCommand[] structureElements = null;
            StructureElement[] approvedStructureElements = null;
            StructureElement[] deletedStructureElements = null;

            structureElements = new StructureElementCommand[] {
                    new StructureElementCommand(null, Type.SYSTEMGROUP, null,            "name", "SgDa",  "description", "comment"),
                    new StructureElementCommand(null, Type.SYSTEM,      systemGroupUuid, "name", "SysDa", "description", "comment"),
                    new StructureElementCommand(null, Type.SUBSYSTEM,   systemUuid,      "name", "SubDa", "description", "comment"),
                    new StructureElementCommand(null, Type.DISCIPLINE,  null,            "name", "DiDa",  "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICEGROUP, disciplineUuid,  "name", "",      "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICETYPE,  deviceGroupUuid, "name", "DtDa",  "description", "comment")
            };

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgDa",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysDa",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubDa", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiDa",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpDa",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtDa",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgDa",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysDa",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubDa", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiDa",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpDa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtDa",   Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.REJECT,  Boolean.FALSE);

            approvedStructureElements = ITUtilStructureElement.assertCreateApprove(structureElements);
            assertNotNull(approvedStructureElements);
            assertEquals(6, approvedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgDa",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysDa",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubDa", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiDa",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpDa",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtDa",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgDa",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysDa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubDa", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiDa",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpDa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtDa",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.REJECT,  Boolean.FALSE);

            deletedStructureElements = ITUtilStructureElement.assertDelete(approvedStructureElements);
            assertNotNull(deletedStructureElements);
            assertEquals(6, deletedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgDa",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysDa",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubDa", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiDa",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpDa",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtDa",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgDa",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysDa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubDa", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiDa",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpDa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtDa",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.APPROVE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.CANCEL,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.REJECT,  Boolean.TRUE);

            approvedStructureElements = ITUtilStructureElement.assertApprove(deletedStructureElements);
            assertNotNull(approvedStructureElements);
            assertEquals(6, approvedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgDa",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysDa",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubDa", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiDa",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpDa",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtDa",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgDa",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysDa",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubDa", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiDa",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpDa",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtDa",   Boolean.TRUE);

            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.REJECT,  Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void deleteCancel() {
        // purpose
        //     test delete and cancel subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        exists in structure
        //     read        is valid to create
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     read        validate cancel
        //     update      delete structures
        //     patch       approve structures
        //     patch       cancel structures
        //
        // note
        //     create, approve in order to delete, cancel

        try {
            StructureElementCommand[] structureElements = null;
            StructureElement[] approvedStructureElements = null;
            StructureElement[] deletedStructureElements = null;
            StructureElement[] cancelledStructureElements = null;

            structureElements = new StructureElementCommand[] {
                    new StructureElementCommand(null, Type.SYSTEMGROUP, null,            "name", "SgDc",  "description", "comment"),
                    new StructureElementCommand(null, Type.SYSTEM,      systemGroupUuid, "name", "SysDc", "description", "comment"),
                    new StructureElementCommand(null, Type.SUBSYSTEM,   systemUuid,      "name", "SubDc", "description", "comment"),
                    new StructureElementCommand(null, Type.DISCIPLINE,  null,            "name", "DiDc",  "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICEGROUP, disciplineUuid,  "name", "",      "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICETYPE,  deviceGroupUuid, "name", "DtDc",  "description", "comment")
            };

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgDc",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysDc",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubDc", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiDc",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpDc",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtDc",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgDc",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysDc",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubDc", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiDc",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpDc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtDc",   Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.REJECT,  Boolean.FALSE);

            approvedStructureElements = ITUtilStructureElement.assertCreateApprove(structureElements);
            assertNotNull(approvedStructureElements);
            assertEquals(6, approvedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgDc",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysDc",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubDc", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiDc",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpDc",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtDc",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgDc",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysDc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubDc", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiDc",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpDc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtDc",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.REJECT,  Boolean.FALSE);

            deletedStructureElements = ITUtilStructureElement.assertDelete(approvedStructureElements);
            assertNotNull(deletedStructureElements);
            assertEquals(6, deletedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgDc",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysDc",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubDc", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiDc",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpDc",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtDc",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgDc",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysDc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubDc", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiDc",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpDc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtDc",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.APPROVE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.CANCEL,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.REJECT,  Boolean.TRUE);

            cancelledStructureElements = ITUtilStructureElement.assertCancel(deletedStructureElements);
            assertNotNull(cancelledStructureElements);
            assertEquals(6, cancelledStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgDc",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysDc",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubDc", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiDc",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpDc",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtDc",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgDc",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysDc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubDc", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiDc",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpDc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtDc",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(cancelledStructureElements, StructureChoice.REJECT,  Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void deleteReject() {
        // purpose
        //     test delete and reject subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        exists in structure
        //     read        is valid to create
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     read        validate reject
        //     update      delete structures
        //     patch       approve structures
        //     patch       reject structures
        //
        // note
        //     create, approve in order to delete, reject

        try {
            StructureElementCommand[] structureElements = null;
            StructureElement[] approvedStructureElements = null;
            StructureElement[] deletedStructureElements = null;
            StructureElement[] rejectedStructureElements = null;

            structureElements = new StructureElementCommand[] {
                    new StructureElementCommand(null, Type.SYSTEMGROUP, null,            "name", "SgDr",  "description", "comment"),
                    new StructureElementCommand(null, Type.SYSTEM,      systemGroupUuid, "name", "SysDr", "description", "comment"),
                    new StructureElementCommand(null, Type.SUBSYSTEM,   systemUuid,      "name", "SubDr", "description", "comment"),
                    new StructureElementCommand(null, Type.DISCIPLINE,  null,            "name", "DiDr",  "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICEGROUP, disciplineUuid,  "name", "",      "description", "comment"),
                    new StructureElementCommand(null, Type.DEVICETYPE,  deviceGroupUuid, "name", "DtDr",  "description", "comment")
            };

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgDr",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysDr",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubDr", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiDr",      Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpDr",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtDr",   Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgDr",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysDr",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubDr", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiDr",      Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpDr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtDr",   Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElements, StructureChoice.REJECT,  Boolean.FALSE);

            approvedStructureElements = ITUtilStructureElement.assertCreateApprove(structureElements);
            assertNotNull(approvedStructureElements);
            assertEquals(6, approvedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgDr",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysDr",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubDr", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiDr",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpDr",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtDr",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgDr",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysDr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubDr", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiDr",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpDr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtDr",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(approvedStructureElements, StructureChoice.REJECT,  Boolean.FALSE);

            deletedStructureElements = ITUtilStructureElement.assertDelete(approvedStructureElements);
            assertNotNull(deletedStructureElements);
            assertEquals(6, deletedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgDr",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysDr",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubDr", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiDr",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpDr",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtDr",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgDr",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysDr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubDr", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiDr",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpDr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtDr",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.APPROVE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.CANCEL,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(deletedStructureElements, StructureChoice.REJECT,  Boolean.TRUE);

            rejectedStructureElements = ITUtilStructureElement.assertReject(deletedStructureElements);
            assertNotNull(rejectedStructureElements);
            assertEquals(6, rejectedStructureElements.length);

            ITUtilStructureElement.assertExists(Type.SYSTEMGROUP, "SgDr",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SYSTEM,      "SysDr",     Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM,   "Sys-SubDr", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE,  "DiDr",      Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "GrpDr",     Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICETYPE,  "Di-DtDr",   Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEMGROUP, "SgDr",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SYSTEM,      "SysDr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM,   "Sys-SubDr", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE,  "DiDr",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "GrpDr",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICETYPE,  "Di-DtDr",   Boolean.FALSE);

            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.APPROVE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.CANCEL,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(rejectedStructureElements, StructureChoice.REJECT,  Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

}
